"""Module to keep track of type aliases used in Blueprints."""

# <editor-fold desc="COUNTERS">
AMOUNT = int
"""Amount, represented as a int."""
# </editor-fold>

# <editor-fold desc="DISTANCES">
MM = float
"""Millimeters (mm), represented as a float."""
CM = float
"""Centimeters (cm), represented as a float."""
M = float
"""Meters (m), represented as a float."""
KM = float
"""Kilometers (km), represented as a float."""
# </editor-fold>

# <editor-fold desc="ROTATIONS">
DEG = float
"""Degrees (deg), represented as a float."""
RAD = float
"""Radians (rad), represented as a float."""
# </editor-fold>

# <editor-fold desc="WEIGHTS">
G = float
"""Grams (g), represented as a float."""
KG = float
"""Kilograms (kg), represented as a float."""
KG_M3 = float
"""Kilograms per cubic meter (kg/m³), represented as a float."""
KG_M2 = float
"""Kilograms per square meter (kg/m2), represented as a float."""
KG_M = float
"""Kilograms per meter (kg/m), represented as a float."""
# </editor-fold>

# <editor-fold desc="FORCES">
N = float
"""Newtons (N), represented as a float."""
KN = float
"""Kilonewtons (kN), value represented as a float."""
# </editor-fold>

# <editor-fold desc="FORCE PER UNIT LENGTH">
N_MM = float
"""Newtons per millimeter (N/mm), represented as a float."""
N_M = float
"""Newtons per meter (N/m), represented as a float."""
KN_M = float
"""Kilonewtons per meter (kN/m), represented as a float."""
# </editor-fold>

# <editor-fold desc="MOMENTS">
NMM = float
"""Newton-millimeters (Nmm), represented as a float."""
NM = float
"""Newton-meters (Nm), represented as a float."""
KNM = float
"""Kilonewton-meters (kNm), represented as a float."""
# </editor-fold>

# <editor-fold desc="STRESSES">
KPA = float
"""Kilopascals (KPa), represented as a float."""
MPA = float
"""Megapascals (MPa), represented as a float."""
GPA = float
"""Gigapascals (GPa), represented as a float."""
# </editor-fold>

# <editor-fold desc="AREAS">
MM2 = float
"""Square millimeters (mm²), represented as a float."""
MM2_M = float
"""Square millimeters per meter, (mm²/m), represented as a float."""
CM2 = float
"""Square centimeters (cm²), represented as a float."""
DM2 = float
"""Square decimeters (dm²), represented as a float."""
M2 = float
"""Square meters (m²), represented as a float."""
KM2 = float
"""Square kilometers (km²), represented as a float."""
# </editor-fold>

# <editor-fold desc="VOLUMES">
MM3 = float
"""Cubic millimeters (mm³), represented as a float."""
CM3 = float
"""Cubic centimeters (cm³), represented as a float."""
M3 = float
"""Cubic meters (m³), represented as a float."""
M3_M = float
"""Cubic meters per meter (m³/m), represented as a float."""
# </editor-fold>

# <editor-fold desc="BI-QUADRATIC">
MM4 = float
"""Millimeters to the fourth power (mm⁴), represented as a float."""
CM4 = float
"""Centimeters to the fourth power (cm⁴), represented as a float."""
M4 = float
"""Meters to the fourth power (m⁴), represented as a float."""
# </editor-fold>

# <editor-fold desc="COMPOUNDS">
N_M2 = float
"""Newton per square meter (N/m²), represented as a float."""
KN_M2 = float
"""Kilonewton per square meter (kN/m²), represented as a float."""
KN_M3 = float
"""Kilonewton per cubic meter (kN/m³), represented as a float."""
# </editor-fold>

# <editor-fold desc="TIME">
SECONDS = float
"""Seconds (sec), represented as a float."""
MINUTES = float
"""Minutes (min), represented as a float."""
HOURS = float
"""Hours (h), represented as a float."""
DAYS = float
"""Days (days), represented as a float."""
WEEKS = float
"""Weeks (weeks), represented as a float."""
MONTHS = float
"""Months (months), represented as a float."""
YEARS = float
"""Years (years), represented as a float."""
# </editor-fold>

# <editor-fold desc="Percentages">
PERCENTAGE = float
"""Percentage (%), represented as a float."""
PER_MILLE = float
"""Per mille (‰), represented as a float."""
# </editor-fold>

# <editor-fold desc="RATIOS">
RATIO = float
"""Ratio, represented as a float."""
PER_DEGREE = float
"""Per degree (1/°), represented as a float."""
MM_KN = float
"""Ratio of deflection over applied load."""
DIMENSIONLESS = float
"""Dimensionless, represented as a float."""
# </editor-fold>

# <editor-fold desc="FREQUENCY">
HZ = float
"""Frequency (hertz), represented as a float."""
# </editor-fold>

# <editor-fold desc="TEMPERATURE">
DEG_C = float
"""Degrees (Celsius), represented as a float."""
DEG_K = float
"""Degrees (Kelvin), represented as a float."""
# </editor-fold>

# <editor-fold desc="RECIPROCAL">
ONE_OVER_MM = float
"""Reciprocal (1/mm), represented as a float."""
ONE_OVER_M = float
"""Reciprocal (1/m), represented as a float."""
# </editor-fold>

# <editor-fold desc="VELOCITIES">
M_PER_S = float
"""Velocity (m/s), represented as a float."""
KM_PER_H = float
"""Velocity (km/h), represented as a float."""
M_NS2 = float
"""Velocity response of unit impulse load (m/Ns²), represented as a float."""
# </editor-fold>

# <editor-fold desc="ACCELERATIONS">
M_PER_S2 = float
"""Acceleration (m/s²), represented as a float."""
MM_PER_S2 = float
"""Acceleration (mm/s²), represented as a float."""
# </editor-fold>

# <editor-fold desc="STIFFNESSES">
NM2_M = float
"""Bending stiffness (Nm²/m), represented as a float."""
# </editor-fold>
